/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#include "keys.h"
#include <iostream>

using namespace std;

//Key functions

Key::Key (void) :
down(false), pressed(0), last_event(0), done_once(false), key_name(""), delay(0), speed(0) {}

Key::Key (string ikey_name, int idelay, int ispeed) :
down(false), pressed(0), last_event(0), done_once(false), key_name(ikey_name), delay(idelay), speed(ispeed){}

bool Key::is_down (void)
{
	return (down);
}

bool Key::get_event (void)
{
	if (down)
	{
		if (!done_once)
		{
			done_once = true;
			return true;
		} else if (speed != 0 && SDL_GetTicks() >= last_event+(last_event >= pressed+delay ? speed : delay))
		{
			last_event += last_event >= pressed+delay ? speed : delay;
			return true;
		}
	}
	return false;
}

void Key::reset_state (void)
{
	down = false;
}

void Key::reset_repeat (void)
{
	if (down)
	{
		done_once = true;
		pressed = last_event = SDL_GetTicks();
	}
}

void Key::handle_event (SDL_Event *event)
{
	if (event->type == SDL_KEYDOWN && key_name == SDL_GetKeyName(event->key.keysym.sym))
	{
		done_once = false;
		down = true;
		pressed = last_event = SDL_GetTicks();
	} else if (event->type == SDL_KEYUP && key_name == SDL_GetKeyName(event->key.keysym.sym)) {
		down = false;
	}
}

//Key_manager functions

void Key_manager::handle_event (SDL_Event *event)
{
	for (map<string, Key>::iterator i = keys.begin(); i != keys.end(); i++)
	{
		i->second.handle_event(event);
	}
}

void Key_manager::add_key (string name, Key k)
{
	keys.insert(pair<string, Key>(name, k));
}

void Key_manager::remove_key (string name)
{
	keys.erase(keys.find(name));
}

void Key_manager::clear (void)
{
	keys.clear();
}

void Key_manager::reset_state (void)
{
	for (map<string, Key>::iterator i = keys.begin(); i != keys.end(); i++)
	{
		i->second.reset_state();
	}
}

void Key_manager::reset_repeat (void)
{
	for (map<string, Key>::iterator i = keys.begin(); i != keys.end(); i++)
	{
		i->second.reset_repeat();
	}
}

Key& Key_manager::operator[] (const string& name)
{
	return keys[name];
}

