/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#ifndef SPORKTRIS_H
#define SPORKTRIS_H

#include <SDL.h>
#include <vector>
#include <string>
#include <map>
#include "keys.h"
#include "namedv.h"
#include "layers.h"
#include "pieces.h"

using namespace std;

class Sporktris
{
	vector< vector<block_type> > board;
	vector< vector<block_type> > bufboard;
	Namedv *prefs;
	Namedv *user_prefs;
	piece cur_piece, next_piece;
	vector<string> piece_specs;
	SDL_Surface *screen;
	map<block_type, SDL_Surface *> blocks;
	SDL_Surface *game_over;
	SDL_Surface *paused_img;
	SDL_Surface *level_box;
	SDL_Surface *points_box;
	SDL_Surface *lines_box;
	SDL_Surface *level_theme;
	SDL_Surface *next_piece_box;
	SDL_Surface *block_outlines;
	vector<SDL_Surface*> themes;
	map<unsigned char, SDL_Surface*> outline_surfs_b;
	map<unsigned char, SDL_Surface*> outline_surfs_w;
	int theme_id;
	layerset ls;
	Key_manager keys;
	enum {base_speed = 500};
	unsigned int speed;
	bool paused;
	bool awaiting_gravity;
	unsigned long last_step;
	
	void load_surfs (void);
	void pause (void);
	void init_keys (void);
	void init_board (void);
	void init_layers (void);
	void load_pieces (void);
	bool handle_events (SDL_Event *event);
	int handle_input (void);
	void update_win (void);
	void drop_piece (void);
	bool check_line (bool type, int line);
	bool gravity (void);
	int random (int min, int max);
	void inc_speed (void);
	piece rand_piece (void);
	void set_board (int x, int y, block_type val);
	bool turn_piece (bool direction);
	void move_piece (int xChange, int yChange);
	void update_level (void);
	void update_points (void);
	void update_lines (void);
	void blit_nums (int num, SDL_Surface *destSurf);
	void update_next_piece (void);
	void clear_surf (SDL_Surface *surf);
	bool end_game (void);
	void color_blocks (SDL_Surface *block);
	void make_garbage (int height);
	void update_outlines (const vector< pair<int, int> >& to_update);
	
	public:
	Sporktris (Namedv *prefs_ptr, Namedv *user_prefs_ptr);
	~Sporktris (void);
	bool game_loop (void);
	void reset (void);
	void set_level (int newLevel);
	
	int level;
	int points;
	int cleared_lines;
};

#endif
